#include <stdio.h>
#include <string.h>
#include <gmpc/plugin.h>
#include <config.h>

static void wp_add(GtkWidget *cat_tree);
static void wp_selected(GtkWidget *container);
static void wp_unselected(GtkWidget *container);
static void wp_changed(GtkWidget *tree, GtkTreeIter *iter);
static void wp_status_changed(MpdObj *mi, ChangedStatusType what, void *userdata);

static GtkWidget *moz = NULL;
static GtkWidget *wp_vbox = NULL;
static GtkWidget *pgb = NULL;

static void wp_start_init(void);
static int wp_get_enabled();
static void wp_set_enabled(int enabled);

GtkWidget * (*moz_new)(void) = NULL;
GtkWidget * (*moz_load_url)(GtkWidget *moz, const char *url) = NULL;


/* Needed plugin_wp stuff */
gmpcPlBrowserPlugin wp_gbp = {
	.add                    = wp_add,		/** add */
	.selected               = wp_selected,	/** selected */
	.unselected             = wp_unselected,	/** unselected */
	.cat_selection_changed  = wp_changed,	/** changed */
};
int plugin_api_version = PLUGIN_API_VERSION;
gmpcPlugin plugin = {
	.name = "WikiPedia",
	.version        = {PLUGIN_MAJOR_VERSION,PLUGIN_MINOR_VERSION,PLUGIN_MICRO_VERSION},
	.plugin_type = GMPC_PLUGIN_PL_BROWSER,
	.init = wp_start_init, /*init */
	.browser = &wp_gbp,
    .mpd_status_changed = 	wp_status_changed, /* status changed */
	.get_enabled = wp_get_enabled,
	.set_enabled = wp_set_enabled
};


/* Playlist window row reference */
static GtkTreeRowReference *wiki_ref = NULL;

void wp_changed(GtkWidget *tree, GtkTreeIter *iter){
  if(moz_load_url)
  {
    mpd_Song *song = mpd_playlist_get_current_song(connection);
    if(song)
    {
      gchar *artist= NULL;
      /* copied from metadata.h */
			if(song->artist && cfg_get_single_value_as_int_with_default(config, "metadata", "rename", FALSE))
			{
				gchar **str = g_strsplit(song->artist, ",", 2);
				if(str[1]) {
					artist = g_strdup_printf("%s %s", g_strstrip(str[1]), g_strstrip(str[0]));
				}
				g_strfreev(str);
				debug_printf(DEBUG_INFO, "string converted to: '%s'", song->artist);
			}

      if(song->artist != NULL){
        int i;
        gchar *url = g_strdup_printf("http://wikipedia.com/w/index.php?useskin=chick&title=%s", (artist)?artist:song->artist);
        for(i=0;i< strlen(url);i++){
          if(url[i] == ' ') url[i] = '_';
        }
        moz_load_url(moz,url);
        g_free(url);
      }
      else{
        moz_load_url(moz, "http://musicpd.org/");
      }
      if(artist) g_free(artist);
    }
  }
}

void wp_progress(GtkWidget *mozem, int cur, int max)
{
/*	float progress = cur/(float)max;
	printf("%i %i\n", cur,max);
	progress = (progress<0)? 0:(progress>1)? 1:progress;
	if(max == -1)
	{
		gtk_progress_bar_pulse(GTK_PROGRESS_BAR(pgb));
	}
	else
	{
		gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(pgb),progress); 
	}
*/
}
void wp_init()
{
  if(moz_new)
  {
    GtkWidget *sw =gtk_frame_new(NULL);
    gtk_frame_set_shadow_type(GTK_FRAME(sw), GTK_SHADOW_ETCHED_IN);
    wp_vbox = gtk_vbox_new(FALSE, 6);
    moz = moz_new();
    if(moz == NULL)
    {
    }
    gtk_container_add(GTK_CONTAINER(sw), moz);
    gtk_box_pack_start_defaults(GTK_BOX(wp_vbox), sw);
    pgb = gtk_progress_bar_new();
    /*	gtk_box_pack_start(GTK_BOX(wp_vbox), pgb, FALSE, TRUE, 0);*/
    gtk_widget_show_all(wp_vbox);
    g_object_ref(G_OBJECT(wp_vbox));
    g_signal_connect(moz, "progress", G_CALLBACK(wp_progress), NULL);
  }
}

void wp_add(GtkWidget *cat_tree)
{
  if(!moz_new)
    return;
	GtkTreePath *path = NULL;
	GtkTreeStore *pl3_tree = (GtkTreeStore *)gtk_tree_view_get_model(GTK_TREE_VIEW(cat_tree));	
	GtkTreeIter iter;
	if(!cfg_get_single_value_as_int_with_default(config, "wp-plugin", "enable", 0)) return;
	gtk_tree_store_append(pl3_tree, &iter, NULL);
	gtk_tree_store_set(pl3_tree, &iter, 
			PL3_CAT_TYPE, plugin.id,
			PL3_CAT_TITLE, "Wikipedia Lookup",
			PL3_CAT_INT_ID, "/",
			PL3_CAT_ICON_ID, "gtk-info",
			PL3_CAT_PROC, TRUE,
			PL3_CAT_ICON_SIZE,GTK_ICON_SIZE_DND,-1);
	if (wiki_ref)
	{
		gtk_tree_row_reference_free(wiki_ref);
		wiki_ref = NULL;
	}

	path = gtk_tree_model_get_path(GTK_TREE_MODEL(playlist3_get_category_tree_store()), &iter);
	if (path)
	{
		wiki_ref = gtk_tree_row_reference_new(GTK_TREE_MODEL(playlist3_get_category_tree_store()), path);
		gtk_tree_path_free(path);
	}

}

void wp_selected(GtkWidget *container)
{

	if(wp_vbox== NULL)
	{
		wp_init();
	}

	gtk_container_add(GTK_CONTAINER(container), wp_vbox);
	gtk_widget_show_all(wp_vbox);
	while (gtk_events_pending ())
		gtk_main_iteration ();

}

void wp_unselected(GtkWidget *container)
{
	gtk_widget_hide(moz);
	gtk_container_remove(GTK_CONTAINER(container),wp_vbox);
}


void wp_set_enabled(int enabled)
{
	cfg_set_single_value_as_int(config, "wp-plugin", "enable", enabled);
	if (enabled)
	{
		if(wiki_ref == NULL)
		{
			wp_add(GTK_WIDGET(playlist3_get_category_tree_view()));
		}
	}
	else if (wiki_ref)
	{
		GtkTreePath *path = gtk_tree_row_reference_get_path(wiki_ref);
		if (path){
			GtkTreeIter iter;
			if (gtk_tree_model_get_iter(GTK_TREE_MODEL(playlist3_get_category_tree_store()), &iter, path)){
				gtk_tree_store_remove(playlist3_get_category_tree_store(), &iter);
			}
			gtk_tree_path_free(path);
			gtk_tree_row_reference_free(wiki_ref);
			wiki_ref = NULL;
		}                                                                                                  	
	}                                                                                                      	
}

int wp_get_enabled()
{
	return 	cfg_get_single_value_as_int_with_default(config, "wp-plugin", "enable", 0);
}
static void wp_start_init(void)
{

  GModule *handle = g_module_open("libgtkembedmoz.so", G_MODULE_BIND_LOCAL);
  if(!handle)
  {
    debug_printf(DEBUG_ERROR,"Failed to load libgtkembedmoz.so: '%s'\n", g_module_error());
    return;
  }
  if(!g_module_symbol(handle, "gtk_moz_embed_new", (void *)&moz_new) || !g_module_symbol(handle, "gtk_moz_embed_load_url", (void *)&moz_load_url))
  {
    debug_printf(DEBUG_ERROR,"Symbols not available in libgtkembedmoz.so: '%s'\n", g_module_error());
    g_module_close(handle);
    moz_new = NULL;
    moz_load_url = NULL;
  }
}


static void wp_status_changed(MpdObj *mi, ChangedStatusType what, void *userdata)
{
	if(pl3_cat_get_selected_browser() == plugin.id)
	{
		if(what&(MPD_CST_SONGID|MPD_CST_SONGPOS))
		{
			wp_changed(NULL, NULL);	
		}
	}
}

